##
# Pixabay API (unofficial)
# @author Luk� Pleva� <lukas@plevac.eu>
# @date 3.2.2022

import requests
from .params import params
from .query  import query as queryCore
from .image  import image as imageCore
from .video  import video as videoCore

class core:
    ##
    # Init function
    # @param apiKey api key for pixabay API
    # @param host host of api (default: https://pixabay.com/api/)
    # @return core obj
    def __init__(self, apiKey, host='https://pixabay.com/api/'):
        self.apiKey = apiKey
        self.host   = host

    ##
    # Make search qvery on API
    # @param query search words (ex: big tree)
    # @param lang language of search words (default "en") 
    # supported: cs, da, de, en, es, fr, id, it, hu, nl, no, pl, pt, ro, sk, fi, sv, tr, vi, th, bg, ru, el, ja, ko, zh
    # @param orientation orientation of image (default "all")
    # supported: all, horizontal, vertical
    # @param perPage number of images per one request (default: 25)
    # @param order order of images (default: "popular")
    # supported: popular, latest
    # @param safeSearch a flag indicating that only images suitable for all ages should be returned (Default: False)
    # @param minWidth min width of image (default: 0)
    # @param minHeight min height of image (default: 0)
    # @param editorsChoice images that have received an Editor's Choice award (Default: False)
    # @param category filter images by category (default: "all")
    # supported: all, backgrounds, fashion, nature, science, education, feelings, health, people, religion, places, animals, industry, computer, food, sports, transportation, travel, buildings, business, music
    # @param colors filter images by color properties. A comma separated list of values may be used to select multiple properties. (Default: "all")
    # supported: all, grayscale, transparent, red, orange, yellow, green, turquoise, blue, lilac, pink, white, gray, black, brown 
    #
    # @return query object
    def query(self, query='', lang='en', orientation='all', perPage=50, order="popular", safeSearch=False, minWidth=0, minHeight=0, editorsChoice=False, category='all', colors='all'):
        param = params(
            host          = self.host,
            apiKey        = self.apiKey,
            query         = query,
            lang          = lang,
            orientation   = orientation,
            perPage       = perPage,
            order         = order,
            safeSearch    = safeSearch,
            minWidth      = minWidth,
            minHeight     = minHeight,
            editorsChoice = editorsChoice,
            category      = category,
            colors        = colors
        )

        return queryCore(param)

    ##
    # Get image by ID
    # @param iid image id
    # @return image object
    def image(self, iid):
        uri = "{host}?key={api}&id={id}".format(
            host    = self.host,
            id      = iid,
            api     = self.apiKey,
        )

        r = requests.get(uri)

        if (r.status_code != 200):
            raise ValueError('Pixabay return status code != 200 for uri', uri, 'Invalid parameters?')

        return imageCore(r.json()['hits'][0])

##
    # Get video by ID
    # @param iid video id
    # @return video object
    def video(self, iid):
        uri = "{host}videos/?key={api}&id={id}".format(
            host    = self.host,
            id      = iid,
            api     = self.apiKey,
        )

        r = requests.get(uri)

        if (r.status_code != 200):
            raise ValueError('Pixabay return status code != 200 for uri', uri, 'Invalid parameters?')

        return videoCore(r.json()['hits'][0])

    # @return query object
    def queryVideo(self, query='', lang='en', orientation='all', perPage=50, order="popular", safeSearch=False, minWidth=0, minHeight=0, editorsChoice=False, category='all', colors='all'):
        param = params(
            host          = self.host+'videos/',
            apiKey        = self.apiKey,
            query         = query,
            lang          = lang,
            orientation   = orientation,
            perPage       = perPage,
            order         = order,
            safeSearch    = safeSearch,
            minWidth      = minWidth,
            minHeight     = minHeight,
            editorsChoice = editorsChoice,
            category      = category,
            colors        = colors
        )

        return queryCore(param)


##
# Pixabay API (unofficial)
# @author Luká¹ Plevaè <lukas@plevac.eu>
# @date 17.9.2022

from datetime import datetime, timezone
import requests
import re

class image:
    ##
    # Init image object
    # @param data data of image in JSON format
    # @return image obj
    def __init__(self, data):
        self._raw_data = data

    ##
    # Get ID of image
    def getId(self):
        return self._raw_data['id']
    
    ##
    # Get page url of image
    def getPageURL(self):
        return self._raw_data['pageURL']

    ##
    # Get type of image
    def getType(self):
        return self._raw_data['type']
    
    ##
    # Get tags of image
    def getTags(self):
        return self._raw_data['tags']

    ##
    # Get url of Preview
    def getPreviewURL(self):
        return self._raw_data['previewURL']

    ##
    # Get width of Preview
    def getPreviewWidth(self):
        return self._raw_data['previewWidth']

    ##
    # Get Height of Preview
    def getPreviewHeight(self):
        return self._raw_data['previewHeight']

    ##
    # Get webformat URL
    def getWebformatURL(self):
        return self._raw_data['webformatURL']

    ##
    # Get width of webformat
    def getWebformatWidth(self):
        return self._raw_data['webformatWidth']

    ##
    # Get height of webformat
    def getWebformatHeight(self):
        return self._raw_data['webformatHeight']

    ##
    # Get large Image URL
    def getLargeImageURL(self):
        return self._raw_data['largeImageURL']

    ##
    # Get width of image
    def getImageWidth(self):
        return self._raw_data['imageWidth']

    ##
    # Get height of image
    def getImageHeight(self):
        return self._raw_data['imageHeight']
    
    ##
    # Get size of image
    def getImageSize(self):
        return self._raw_data['imageSize']
    
    ##
    # Get views of image
    def getViews(self):
        return self._raw_data['views']

    ##
    # Get downloads of image
    def getDownloads(self):
        return self._raw_data['downloads']

    ##
    # Get collections of image
    def getCollections(self):
        return self._raw_data['collections']

    ##
    # Get likes of image
    def getLikes(self):
        return self._raw_data['likes']

    ##
    # Get comments of image
    def getComments(self):
        return self._raw_data['comments']

    ##
    # Get author id of image
    def getUserId(self):
        return self._raw_data['user_id']
    
    ##
    # Get name of author of image
    def getUser(self):
        return self._raw_data['user']

    ##
    # Get url of image of author of image
    def getUserImageURL(self):
        return self._raw_data['userImageURL']

    ##
    # Download image to varaible
    # @param imtype type if image (webformat, preview, largeImage) (Default: webformat)
    # @return byte array of image
    def downloadRaw(self, imtype = 'webformat'):
        
        uri = None

        if (imtype == 'webformat'):
            uri = self.getWebformatURL()
        elif (imtype == 'largeImage'):
            uri = self.getLargeImageURL()
        elif (imtype == 'preview'):
            uri = self.getPreviewURL()
        else:
            raise ValueError('supported types is webformat, largeImage and preview.', imtype, 'unsupported')

        r = requests.get(uri, allow_redirects=True)

        if (r.status_code != 200):
            raise ValueError('Pixabay return status code != 200 for uri', uri, 'Invalid parameters?')

        return r.content

    ##
    # Download image to file
    # @param dst location of file to save
    # @param imtype type if image (webformat, preview, largeImage) (Default: webformat)
    def download(self, dst, imtype = 'webformat'):
        with open(dst, 'wb') as handler:
            handler.write(self.downloadRaw(imtype))
    ##
    # Get published date of image
    # @return datetime UTC of image publication
    def getPublishedDate(self):
        preview_url = self.getPreviewURL()
        
        #  year / month / day / hour / minute
        match = re.search('\d{4}/\d{2}/\d{2}/\d{2}/\d{2}', preview_url)
        if match:
            parts = match.group().split('/')
            return datetime(int(parts[0]), int(parts[1]), int(parts[2]), int(parts[3]), int(parts[4]), tzinfo=timezone.utc)
        
        #  year / month / day
        match = re.search('\d{4}/\d{2}/\d{2}', preview_url)
        if match:
            parts = match.group().split('/')
            return datetime(int(parts[0]), int(parts[1]), int(parts[2]), tzinfo=timezone.utc)



##
# Pixabay API (unofficial)
# @author Luk� Pleva� <lukas@plevac.eu>
# @date 3.2.2022

import requests
from .image import image
from .video import video
import urllib.parse

class query:

    ##
    # Init query on API
    # @params params params of query instance of class params
    # @return query object 
    def __init__(self, params):
        self.params = params
        self.cache = []
        self._getPage(0)

    ##
    # Get lenght of fonded images
    # @return len of fonded images
    def __len__(self):
        return self.info['totalHits']

    ##
    # Get item on index from result
    # @praram index index of item
    # return image object of item
    def __getitem__(self, index):
        if (index < 0):
            index += self.__len__()

        if not(self._inCache(index)):
            self._addToChache(index)
        if 'videos' in self.params.host:
            return video(self.cache[index])
        else:
            return image(self.cache[index])

    ##
    # Check if is image with index in chace
    # @param index index of image
    # @return bool
    def _inCache(self, index):
        return len(self.cache) > index and self.cache[index] != None

    ##
    # Insert image in cache
    # @param index index of image
    # @param data data of image 
    def _cacheInsert(self, index, data):
        
        for i in range(len(self.cache) - 1, index - 1):
            self.cache.insert(i, None)

        self.cache.insert(index, data)

    ##
    # Download page from API and save to cache
    # @param page index of page (from 0)
    def _getPage(self, page):
        uri = "{host}?key={api}&q={query}&lang={lang}&orientation={orient}&per_page={per}&page={page}&order={order}&safesearch={safe}&min_width={width}&min_height={height}&editors_choice={editors}&category={cat}&colors={colors}".format(
            host    = self.params.host,
            query   = urllib.parse.quote(self.params.query, safe=''),
            api     = self.params.apiKey,
            lang    = self.params.lang,
            orient  = self.params.orientation,
            per     = self.params.perPage,
            page    = page + 1,
            order   = self.params.order,
            safe    = self.params.safeSearch,
            width   = self.params.minWidth,
            height  = self.params.minHeight,
            editors = self.params.editorsChoice,
            cat     = self.params.category,
            colors  = self.params.colors
        )

        r = requests.get(uri)

        if (r.status_code != 200):
            raise ValueError('Pixabay return status code != 200 for uri', uri, 'Invalid parameters?')

        data = r.json()

        # Update info
        self.info              = {}
        self.info['total']     = data['total']
        self.info['totalHits'] = data['totalHits']

        # copy data
        for i in range(len(data['hits'])):
            self._cacheInsert(page * self.params.perPage + i, data['hits'][i])
    
    ##
    # Download image with index to cache
    # @param index index of image
    def _addToChache(self, index):
        page = index // self.params.perPage
        self._getPage(page)


##
# Pixabay API (unofficial)
# @author Lukáš Plevač <lukas@plevac.eu>
# @date 3.2.2022

class params:
    ##
    # Init params object
    # @param host host of API (default: https://pixabay.com/api/)
    # @param query search words (ex: big tree)
    # @param apiKey api key for API
    # @param lang language of search words (default "en") 
    # supported: cs, da, de, en, es, fr, id, it, hu, nl, no, pl, pt, ro, sk, fi, sv, tr, vi, th, bg, ru, el, ja, ko, zh
    # @param orientation orientation of image (default "all")
    # supported: all, horizontal, vertical
    # @param perPage number of images per one request (default: 25)
    # @param order order of images (default: "popular")
    # supported: popular, latest
    # @param safeSearch a flag indicating that only images suitable for all ages should be returned (Default: False)
    # @param minWidth min width of image (default: 0)
    # @param minHeight min height of image (default: 0)
    # @param editorsChoice images that have received an Editor's Choice award (Default: False)
    # @param category filter images by category (default: "all")
    # supported: all, backgrounds, fashion, nature, science, education, feelings, health, people, religion, places, animals, industry, computer, food, sports, transportation, travel, buildings, business, music
    # @param colors filter images by color properties. A comma separated list of values may be used to select multiple properties. (Default: "all")
    # supported: all, grayscale, transparent, red, orange, yellow, green, turquoise, blue, lilac, pink, white, gray, black, brown 
    #
    # @return params object
    def __init__(self, host='https://pixabay.com/api/', query='', apiKey='', lang='en', orientation='all', perPage=25, order="popular", safeSearch=False, minWidth=0, minHeight=0, editorsChoice=False, category='all', colors='all', id=-1):
        self.host          = host
        self.query         = query
        self.apiKey        = apiKey
        self.lang          = lang
        self.orientation   = orientation
        self.perPage       = perPage
        self.order         = order
        self.safeSearch    = 'true' if safeSearch else 'false'
        self.minWidth      = minWidth
        self.minHeight     = minHeight
        self.editorsChoice = 'true' if editorsChoice else 'false'
        self.category      = category
        self.colors        = colors
        self.id            = -1


##
# Pixabay API (unofficial)
# @author qualityassurance
# @email qualityassurance21@163.com
# @date 2022.9.17

import requests

class video:
    ##
    # Init video object
    # @param data data of video in JSON format
    # @return video obj
    def __init__(self, data):
        self._raw_data = data

    ##
    # Get ID of video
    def getId(self):
        return self._raw_data['id']
    
    ##
    # Get page url of video
    def getPageURL(self):
        return self._raw_data['pageURL']

    ##
    # Get type of video
    def getType(self):
        return self._raw_data['type']
    
    ##
    # Get tags of video
    def getTags(self):
        return self._raw_data['tags']

    ##
    # Get url of Preview
    def getDuration(self):
        return self._raw_data['duration']

    ##
    # Get width of Preview
    def getPictureId(self):
        return self._raw_data['picture_id']

    ##
    # Get Height of Preview
    def getVideoLarge(self):
        return self._raw_data['videos']['large']['url']

    ##
    # Get Height of Preview
    def getVideoSmall(self):
        return self._raw_data['videos']['small']['url']

    ##
    # Get Height of Preview
    def getVideoMedium(self):
        return self._raw_data['videos']['medium']['url']

    ##
    # Get Height of Preview
    def getVideoTiny(self):
        return self._raw_data['videos']['tiny']['url']
    
    ##
    # Get views of video
    def getViews(self):
        return self._raw_data['views']

    ##
    # Get downloads of video
    def getDownloads(self):
        return self._raw_data['downloads']

    ##
    # Get likes of video
    def getLikes(self):
        return self._raw_data['likes']

    ##
    # Get comments of video
    def getComments(self):
        return self._raw_data['comments']

    ##
    # Get author id of video
    def getUserId(self):
        return self._raw_data['user_id']
    
    ##
    # Get name of author of video
    def getUser(self):
        return self._raw_data['user']

    ##
    # Get url of video of author of video
    def getUservideoURL(self):
        return self._raw_data['uservideoURL']

    ##
    # Download video to varaible
    # @param imtype type if video (webformat, preview, largevideo) (Default: webformat)
    # @return byte array of video
    def downloadRaw(self, imtype = 'medium'):
        
        uri = None

        if (imtype == 'large'):
            uri = self.getVideoLarge()
        elif (imtype == 'medium'):
            uri = self.getVideoMedium()
        elif (imtype == 'small'):
            uri = self.getVideoSmall()
        elif (imtype == 'tiny'):
            uri = self.getVideoTiny()
        else:
            raise ValueError('supported types is large, medium, small and tiny.', imtype, 'unsupported')

        r = requests.get(uri, allow_redirects=True)

        if (r.status_code != 200):
            raise ValueError('Pixabay return status code != 200 for uri', uri, 'Invalid parameters?')

        return r.content

    ##
    # Download video to file
    # @param dst location of file to save
    def download(self, dst, imtype = 'medium'):
        with open(dst, 'wb') as handler:
            handler.write(self.downloadRaw(imtype))

        


import setuptools
from shutil import copyfile

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pixabay",
    version="0.0.5",
    author="Lukáš Plevač",
    author_email="lukas@plevac.eu",
    description="Unofficial python pixabay API client",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Lukas0025/python-pixabay",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.0',
    install_requires=[
	    'requests'
    ],
)
